#!/usr/bin/env python
from simple_at import *

def run(acc):
  # try to query to the collection interface on the app
  col = acc.queryInterface('IDL:Accessibility/Collection:1.0')
  if col is None:
    raise NotImplementedError('%s does not implement collection' % acc.name)

  # define roles to collect
  # I believe a Python list of roles will be marshalled properly since the doc
  # at http://gnome.org/%7Ebillh/at-spi-new-idl/html/html/namespaceAccessibility.html#a2
  # says it is a sequence<Role>
  roles = [Accessibility.ROLE_PUSH_BUTTON]

  # create a match rule for our query, making invalid params None and MATCH_ANY
  rule = col.createMatchRule(None, Accessibility.MATCH_ANY, 
                           None, Accessibility.MATCH_ANY,
                           roles, Acccessibility.MATCH_ANY,
                           None, Accessibility.MATCH_ANY,
                           False) # don't negate the logic

  # get the first few, recursing into descendants past the immediate children
  # not sure how an inOut param will be treated by pyORBit; guessing that the 
  # backfilled param will be returned as a second return value in Python
  buttons, total = col.getChildren(rule, Accessibility.SORT_ORDER_CANONICAL, True, 5)

  # check if unbounded
  if total == -1:
    print 'collection unbounded'

  # keep track of how many we got back
  size = len(buttons)

  # fetch more buttons if we don't have them all
  if size != total:
    # start at the last returned button and get all that remain
    more, total = col.getNextChildren(buttons[-1], rule, Accessibility.SORT_ORDER_CANONICAL, 
                                    True, total-size)
    buttons += more
    
  # free the match rule
  col.freeMatchRule(rule)
  
  # these should be all the buttons
  return buttons

if __name__ == '__main__':
  d = registry.getDesktop(0)
  for i in xrange(d.childCount):
    c = d.getChildAtIndex(i)
    if c is not None and c.name == 'gcalctool':
      run(c)
      return
  raise RuntimeError('start gcalc')
